@@ -46,6 +46,10 @@ class TestEnforcer(base.BaseTestCase):
             group='oslo_limit',
             auth_type='password'
         )
+        self.config_fixture.config(
+            group='oslo_limit',
+            endpoint_id='ENDPOINT_ID'
+        )
         opts.register_opts(CONF)
         self.config_fixture.config(
             group='oslo_limit',
@@ -223,6 +227,12 @@ class TestEnforcer(base.BaseTestCase):
 class TestFlatEnforcer(base.BaseTestCase):
     def setUp(self):
         super(TestFlatEnforcer, self).setUp()
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
+        self.config_fixture.config(
+            group='oslo_limit',
+            endpoint_id='ENDPOINT_ID'
+        )
+        opts.register_opts(CONF)
         self.mock_conn = mock.MagicMock()
         limit._SDK_CONNECTION = self.mock_conn
 
@@ -259,7 +269,7 @@ class TestFlatEnforcer(base.BaseTestCase):
         enforcer = limit._FlatEnforcer(mock_usage)
         enforcer.enforce(project_id, deltas)
 
-        self.mock_conn.get_endpoint.assert_called_once_with(None)
+        self.mock_conn.get_endpoint.assert_called_once_with('ENDPOINT_ID')
         mock_get_limits.assert_called_once_with(project_id, ["a", "b"])
         mock_usage.assert_called_once_with(project_id, ["a", "b"])
 
@@ -310,6 +320,12 @@ class TestFlatEnforcer(base.BaseTestCase):
 class TestEnforcerUtils(base.BaseTestCase):
     def setUp(self):
         super(TestEnforcerUtils, self).setUp()
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
+        self.config_fixture.config(
+            group='oslo_limit',
+            endpoint_id='ENDPOINT_ID'
+        )
+        opts.register_opts(CONF)
         self.mock_conn = mock.MagicMock()
         limit._SDK_CONNECTION = self.mock_conn
 
@@ -320,7 +336,7 @@ class TestEnforcerUtils(base.BaseTestCase):
         utils = limit._EnforcerUtils()
 
         self.assertEqual(fake_endpoint, utils._endpoint)
-        self.mock_conn.get_endpoint.assert_called_once_with(None)
+        self.mock_conn.get_endpoint.assert_called_once_with('ENDPOINT_ID')
 
     def test_get_registered_limit_empty(self):
         self.mock_conn.registered_limits.return_value = iter([])
@@ -399,6 +415,48 @@ class TestEnforcerUtils(base.BaseTestCase):
         limits = utils.get_project_limits(project_id, ["c", "d"])
         self.assertEqual([('c', 0), ('d', 0)], limits)
 
+    def test__get_project_limit_cache(self, cache=True):
+        # Registered limit = 5 and project limit = 3
+        project_id = uuid.uuid4().hex
+        fix = self.useFixture(
+            fixture.LimitFixture({'foo': 5}, {project_id: {'foo': 3}}))
+
+        utils = limit._EnforcerUtils(cache=cache)
+        foo_limit = utils._get_project_limit(project_id, 'foo')
+
+        self.assertEqual(3, foo_limit.resource_limit)
+        self.assertEqual(1, fix.mock_conn.limits.call_count)
+
+        # Second call should be cached, so call_count for project limits should
+        # remain 1. When cache is disabled, it should increase to 2
+        foo_limit = utils._get_project_limit(project_id, 'foo')
+        count = 1 if cache else 2
+        self.assertEqual(count, fix.mock_conn.limits.call_count)
+
+    def test__get_project_limit_cache_no_cache(self):
+        self.test__get_project_limit_cache(cache=False)
+
+    def test__get_registered_limit_cache(self, cache=True):
+        # Registered limit = 5 and project limit = 3
+        project_id = uuid.uuid4().hex
+        fix = self.useFixture(
+            fixture.LimitFixture({'foo': 5}, {project_id: {'foo': 3}}))
+
+        utils = limit._EnforcerUtils(cache=cache)
+        foo_limit = utils._get_registered_limit('foo')
+
+        self.assertEqual(5, foo_limit.default_limit)
+        self.assertEqual(1, fix.mock_conn.registered_limits.call_count)
+
+        # Second call should be cached, so call_count for project limits should
+        # remain 1. When cache is disabled, it should increase to 2
+        foo_limit = utils._get_registered_limit('foo')
+        count = 1 if cache else 2
+        self.assertEqual(count, fix.mock_conn.registered_limits.call_count)
+
+    def test__get_registered_limit_cache_no_cache(self):
+        self.test__get_registered_limit_cache(cache=False)
+
     def test_get_limit_cache(self, cache=True):
         # No project limit and registered limit = 5
         fix = self.useFixture(fixture.LimitFixture({'foo': 5}, {}))
